package com.persagy.apm.report.indicator.factory;

import com.persagy.apm.report.common.constants.DataConstants;
import com.persagy.apm.report.common.utils.DataUtils;
import com.persagy.apm.report.common.utils.DateUtils;
import com.persagy.apm.report.dependencies.manualfilling.enums.ManualFillingDataTypeEnum;
import com.persagy.apm.report.detail.enums.AttrValueTypeEnum;
import com.persagy.apm.report.detail.model.vo.AttrValueVO;
import com.persagy.apm.report.detail.model.vo.ItemVO;
import com.persagy.apm.report.indicator.codes.budget.BudgetContentCodes;
import com.persagy.apm.report.indicator.manualfilling.model.ManualFillingData;
import com.persagy.apm.report.indicator.manualfilling.model.dto.QueryManualFillingDataDTO;
import com.persagy.apm.report.indicator.manualfilling.service.IManualFillingDataService;
import com.persagy.apm.report.indicator.model.ConvertDefineTool;
import com.persagy.apm.report.indicator.model.entity.BudgetDefine;
import com.persagy.apm.report.indicator.model.entity.EnergyUsageDefine;
import com.persagy.apm.report.userconfig.function.model.Function;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 能耗预算工厂
 *
 * @author lixing
 * @version V1.0 2021/7/20 3:55 下午
 **/
@Service
public class EnergyBudgetFactory {
    @Autowired
    EnergyUsageFactory energyUsageFactory;
    @Autowired
    IManualFillingDataService manualFillingDataService;

    /**
     * 获取当月用量指标
     *
     * @param budgetDefine 预算指标定义
     * @param projectId    项目id
     * @param reportMonth  报告月份
     * @return 当月用量指标
     * @author lixing
     * @version V1.0 2021/7/23 11:09 上午
     */
    public AttrValueVO getCurrentMonth(BudgetDefine budgetDefine, String projectId, Date reportMonth) {
        EnergyUsageDefine energyUsageDefine = ConvertDefineTool.INSTANCE.convert2EnergyUsageDefine(budgetDefine);
        Double currentMonth = energyUsageFactory.getMonthlyUsageValue(energyUsageDefine, reportMonth, projectId);
        String currentMonthStr = currentMonth == null ? null : String.valueOf(currentMonth);
        return new AttrValueVO(BudgetContentCodes.currentMonth.name(),
                currentMonthStr, AttrValueTypeEnum.number_1);
    }

    /**
     * 获取当月预算
     *
     * @param functionId  指标id
     * @param projectId   项目id
     * @param reportMonth 报告月份
     * @return 当月预算
     * @author lixing
     * @version V1.0 2021/7/23 11:09 上午
     */
    public AttrValueVO getMonthlyBudget(String functionId, String projectId, Date reportMonth) {
        String monthlyBudget = getMonthlyBudgetValue(functionId, projectId, reportMonth);
        return new AttrValueVO(BudgetContentCodes.monthlyBudget.name(),
                monthlyBudget, AttrValueTypeEnum.number_1);
    }

    /**
     * 获取本年预算
     *
     * @param functionId  指标id
     * @param projectId   项目id
     * @param reportMonth 报告月份
     * @return 当月预算
     * @author lixing
     * @version V1.0 2021/7/23 11:09 上午
     */
    public AttrValueVO getAnnualBudget(String functionId, String projectId, Date reportMonth) {
        String annualBudgetValue = getAnnualBudgetValue(functionId, projectId, reportMonth);
        return new AttrValueVO(BudgetContentCodes.annualBudget.name(),
                annualBudgetValue, AttrValueTypeEnum.number_1);
    }

    /**
     * 获取当月预算的值
     *
     * @param functionId  指标id
     * @param projectId   项目id
     * @param reportMonth 报告月份
     * @return 当月预算
     * @author lixing
     * @version V1.0 2021/7/23 11:09 上午
     */
    private String getMonthlyBudgetValue(String functionId, String projectId, Date reportMonth) {
        // 获取预算金额
        QueryManualFillingDataDTO queryManualFillingDataDTO = new QueryManualFillingDataDTO();
        queryManualFillingDataDTO.setProjectId(projectId);
        queryManualFillingDataDTO.setFunctionId(functionId);
        queryManualFillingDataDTO.setDataTime(reportMonth);
        queryManualFillingDataDTO.setType(ManualFillingDataTypeEnum.ENERGY_BUDGET.getType());
        List<ManualFillingData> manualFillingDataList = manualFillingDataService.
                queryManualFillingDataList(queryManualFillingDataDTO);
        String monthlyBudget = null;
        if (!CollectionUtils.isEmpty(manualFillingDataList)) {
            monthlyBudget = manualFillingDataList.get(0).getValue();
        }
        return monthlyBudget;
    }

    /**
     * 获取当年预算的值
     *
     * @param functionId  指标id
     * @param projectId   项目id
     * @param reportMonth 报告月份
     * @return 当月预算
     * @author lixing
     * @version V1.0 2021/7/23 11:09 上午
     */
    private String getAnnualBudgetValue(String functionId, String projectId, Date reportMonth) {
        // 获取报告当年所有月份
        List<Date> firstDayOfEveryMonth = DateUtils.getFirstDayOfEveryMonth(
                DateUtils.getFirstDayOfYear(reportMonth),
                DateUtils.getFirstDayOfMonth(reportMonth));
        if (CollectionUtils.isEmpty(firstDayOfEveryMonth)) {
            return null;
        }
        BigDecimal result = null;
        for (Date date : firstDayOfEveryMonth) {
            String monthlyBudgetValue = getMonthlyBudgetValue(functionId, projectId, date);
            if (StringUtils.isBlank(monthlyBudgetValue)) {
                continue;
            }
            if (result == null) {
                result = new BigDecimal("0");
            }
            result = result.add(new BigDecimal(monthlyBudgetValue));
        }
        return result == null ? null : result.toString();
    }


    /**
     * 获取指标本年累计用量
     *
     * @param budgetDefine 预算指标定义
     * @param projectId    项目id
     * @param reportMonth  报告月份
     * @return 本年累计用量
     * @author lixing
     * @version V1.0 2021/7/23 11:09 上午
     */
    public AttrValueVO getAggregate(BudgetDefine budgetDefine, String projectId, Date reportMonth) {
        EnergyUsageDefine energyUsageDefine = ConvertDefineTool.INSTANCE.convert2EnergyUsageDefine(budgetDefine);
        Double aggregateUsageValue = energyUsageFactory.getAggregateUsageValue(energyUsageDefine, reportMonth, projectId);
        String aggregateUsageValueStr = aggregateUsageValue == null ? null : String.valueOf(aggregateUsageValue);
        return new AttrValueVO(BudgetContentCodes.aggregate.name(),
                aggregateUsageValueStr, AttrValueTypeEnum.number_1);
    }

    /**
     * 获取用量和预算的比例
     *
     * @param usage  用量
     * @param budget 预算
     * @return 用量预算比
     * @author lixing
     * @version V1.0 2021/7/27 2:38 下午
     */
    private Double getProportion(Double usage, Double budget) {
        if (usage != null && budget != null && budget != 0d) {
            return DataUtils.doubleDivide(usage, budget);
        }
        return null;
    }

    /**
     * 获取单月用量和预算比
     *
     * @param currentMonth  本月用量
     * @param monthlyBudget 本月预算用量
     * @return 本月用量预算比
     * @author lixing
     * @version V1.0 2021/7/27 2:41 下午
     */
    public AttrValueVO getMonthlyProportion(Double currentMonth, Double monthlyBudget) {
        Double proportion = getProportion(currentMonth, monthlyBudget);
        String proportionStr = proportion == null ? null : String.valueOf(proportion);
        return new AttrValueVO(BudgetContentCodes.monthlyProportion.name(),
                proportionStr, AttrValueTypeEnum.percent_1, DataConstants.PERCENT);
    }

    /**
     * 获取本年用量和预算比
     *
     * @param aggregate    本年累计用量
     * @param annualBudget 本年预算用量
     * @return 本年用量预算比
     * @author lixing
     * @version V1.0 2021/7/27 2:41 下午
     */
    public AttrValueVO getAnnualProportion(Double aggregate, Double annualBudget) {
        Double proportion = getProportion(aggregate, annualBudget);
        String proportionStr = proportion == null ? null : String.valueOf(proportion);
        return new AttrValueVO(BudgetContentCodes.annualProportion.name(),
                proportionStr, AttrValueTypeEnum.percent_1, DataConstants.PERCENT);
    }

    /**
     * 获取预算信息
     *
     * @param function     指标对象
     * @param budgetDefine 预算指标定义
     * @param projectId    项目id
     * @param reportMonth  报告月份
     * @return 预算信息
     * @author lixing
     * @version V1.0 2021/7/22 6:39 下午
     */
    public ItemVO getBudgetInfo(Function function, BudgetDefine budgetDefine, String projectId, Date reportMonth) {
        List<AttrValueVO> attrValues = new ArrayList<>();
        ItemVO itemVO = new ItemVO(function.getId(), function.getName(), attrValues);

        // 本月能耗
        AttrValueVO currentMonth = getCurrentMonth(budgetDefine, projectId, reportMonth);
        attrValues.add(currentMonth);
        // 本月预算
        AttrValueVO monthlyBudget = getMonthlyBudget(function.getId(), projectId, reportMonth);
        attrValues.add(monthlyBudget);
        // 本月能耗预算比
        AttrValueVO monthlyProportion = getMonthlyProportion(
                DataUtils.parseDouble(currentMonth.getValue()), DataUtils.parseDouble(monthlyBudget.getValue()));
        attrValues.add(monthlyProportion);
        // 本年累计能耗
        AttrValueVO aggregate = getAggregate(budgetDefine, projectId, reportMonth);
        attrValues.add(aggregate);
        // 本年预算
        AttrValueVO annualBudget = getAnnualBudget(function.getId(), projectId, reportMonth);
        attrValues.add(annualBudget);
        // 本年能耗预算比
        AttrValueVO annualProportion = getAnnualProportion(
                DataUtils.parseDouble(aggregate.getValue()), DataUtils.parseDouble(annualBudget.getValue()));
        attrValues.add(annualProportion);

        return itemVO;
    }
}
