package com.persagy.apm.report.paragraphs.service.impl;

import com.persagy.apm.report.common.utils.ApmStrUtils;
import com.persagy.apm.report.userconfig.functiongroup.model.FunctionGroup;
import com.persagy.apm.report.userconfig.functiongroup.service.IFunctionGroupService;
import com.persagy.apm.report.userconfig.paragraph.model.Paragraph;
import com.persagy.apm.report.userconfig.paragraph.service.IParagraphService;
import com.persagy.apm.report.userconfig.paragraphcustom.service.IParagraphCustomService;
import com.persagy.apm.report.userconfig.pargraphdefine.model.ParagraphDefine;
import com.persagy.apm.report.userconfig.pargraphdefine.service.IParagraphDefineService;
import com.persagy.apm.report.userconfig.reporttype.service.IReportTypeService;
import com.persagy.apm.report.detail.enums.ParagraphContentItemTypeEnum;
import com.persagy.apm.report.detail.model.vo.AttrValueVO;
import com.persagy.apm.report.detail.model.vo.GroupVO;
import com.persagy.apm.report.detail.model.vo.ItemVO;
import com.persagy.apm.report.detail.paragraphvalue.model.vo.ParagraphContentVO;
import com.persagy.apm.report.detail.paragraphvalue.service.IReportParagraphValueService;
import com.persagy.apm.report.outline.model.ReportOutline;
import com.persagy.apm.report.outline.service.IReportOutlineService;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 段落建造者实现类
 *
 * @author lixing
 * @version V1.0 2021/7/20 9:42 上午
 **/
@Service
public abstract class ProjectParagraphBuilder extends ParagraphBuilder {
    @Autowired
    IParagraphDefineService paragraphDefineService;
    @Autowired
    IReportOutlineService reportOutlineService;
    @Autowired
    IFunctionGroupService functionGroupService;
    @Autowired
    IParagraphCustomService paragraphCustomService;
    @Autowired
    IReportTypeService reportTypeService;
    @Autowired
    IReportParagraphValueService paragraphValueService;
    @Autowired
    IParagraphService paragraphService;

    /**
     * 计算段落内容部分
     *
     * @param paragraph 段落对象
     * @param reportOutline 报告概要对象
     * @return 段落内容部分
     * @author lixing
     * @version V1.0 2021/8/2 9:43 上午
     */
    @Override
    public ParagraphContentVO getParagraphContent(Paragraph paragraph, ReportOutline reportOutline) {
        String projectId = reportOutline.getBelong();
        Date reportMonth = reportOutline.getReportMonth();
        return getParagraphContentVO(paragraph, projectId, reportMonth);
    }

    /**
     * 计算段落内容部分
     *
     * @param paragraph 段落对象
     * @param projectId 项目id
     * @param reportMonth 报告月份
     * @return 段落内容部分
     * @author lixing
     * @version V1.0 2021/8/2 9:43 上午
     */
    public ParagraphContentVO getParagraphContentVO(Paragraph paragraph, String projectId, Date reportMonth) {
        // 获取段落的分组和条目信息
        String defineId = paragraph.getDefineId();
        ParagraphDefine paragraphDefine = paragraphDefineService.queryParagraphDefineDetail(defineId);
        String groupIds = paragraphDefine.getGroupIds();
        String itemIds = paragraphDefine.getItemIds();
        List<String> groupIdList = ApmStrUtils.str2List(groupIds);
        List<String> itemIdList = ApmStrUtils.str2List(itemIds);

        ParagraphContentVO paragraphContentVO = new ParagraphContentVO();
        List<ItemVO> items = new ArrayList<>();
        List<GroupVO> groups = new ArrayList<>();

        if (itemIdList != null) {
            for (String itemId : itemIdList) {
                // 获取条目的取值
                ItemVO projectItemValue = getProjectItemValue(itemId, projectId, reportMonth);
                items.add(projectItemValue);
            }
        }

        if (groupIdList != null) {
            for (String groupId : groupIdList) {
                // 获取分组的取值
                GroupVO groupValue = getProjectGroupValue(groupId, projectId, reportMonth);
                groups.add(groupValue);
            }
        }

        paragraphContentVO.setItems(items);
        paragraphContentVO.setGroups(groups);
        paragraphContentVO.setSummary(getSummary(groups, items, Lists.newArrayList(projectId)));
        paragraphContentVO.setItemType(ParagraphContentItemTypeEnum.FUNCTION);
        return paragraphContentVO;
    }

    /**
     * 获取项目上分组的取值
     *
     * @param groupId     分组id
     * @param projectId   项目id
     * @param reportMonth 报告月份
     * @return 条目取值对象
     * @author lixing
     * @version V1.0 2021/7/21 10:06 上午
     */
    public GroupVO getProjectGroupValue(String groupId, String projectId, Date reportMonth) {
        GroupVO groupVO = new GroupVO();
        groupVO.setId(groupId);
        // 获取分组名称
        FunctionGroup functionGroup = functionGroupService.queryFunctionGroupDetail(groupId);
        groupVO.setName(functionGroup.getName());

        // 获取分组下条目
        String functionIds = functionGroup.getFunctionIds();
        List<String> functionIdsList = ApmStrUtils.str2List(functionIds);
        List<ItemVO> itemVOList = new ArrayList<>();
        if (!CollectionUtils.isEmpty(functionIdsList)) {
            for (String itemId : functionIdsList) {
                itemVOList.add(getProjectItemValue(itemId, projectId, reportMonth));
            }
            groupVO.setItems(itemVOList);
        }
        // 获取分组下的分组
        String groupIds = functionGroup.getGroupIds();
        List<String> groupIdList = ApmStrUtils.str2List(groupIds);
        List<GroupVO> groupVOList = new ArrayList<>();
        if (!CollectionUtils.isEmpty(groupIdList)) {
            for (String sonGroupId : groupIdList) {
                groupVOList.add(getProjectGroupValue(sonGroupId, projectId, reportMonth));
            }
        }

        groupVO.setGroups(groupVOList);
        groupVO.setShowSummary(functionGroup.getShowSummary());
        groupVO.setSummary(getSummary(groupVOList, itemVOList, Lists.newArrayList(projectId)));
        return groupVO;
    }

    /**
     * 获取项目上条目的取值
     *
     * @param itemId      条目id
     * @param projectId   项目id
     * @param reportMonth 报告月份
     * @return 条目取值对象
     * @author lixing
     * @version V1.0 2021/7/21 10:06 上午
     */
    public abstract ItemVO getProjectItemValue(String itemId, String projectId, Date reportMonth);

    /**
     * 获取单项目取值汇总信息 (大区报告中)
     *
     * @param projectId           项目id
     * @param projectName           项目名称
     * @param projectParagraphContent 项目段落内容信息
     * @return 单项目取值汇总
     * @author lixing
     * @version V1.0 2021/7/21 10:06 上午
     */
    public ItemVO getProjectValue(String projectId, String projectName, ParagraphContentVO projectParagraphContent) {
        ItemVO itemVO = new ItemVO();
        itemVO.setId(projectId);
        itemVO.setName(projectName);
        itemVO.setAttrs(getProjectSummary(projectParagraphContent));
        return itemVO;
    }

    /**
     * 获取单项目汇总属性取值信息 （大区报告中）
     *
     * @param projectParagraphContent 项目段落内容信息
     * @return 单项目汇总属性取值信息
     * @author lixing
     * @version V1.0 2021/7/22 11:36 上午
     */
    public abstract List<AttrValueVO> getProjectSummary(ParagraphContentVO projectParagraphContent);
}
