package com.persagy.apm.report.userconfig.function.service.impl;

import com.persagy.apm.common.context.AppContext;
import com.persagy.apm.report.userconfig.function.dao.FunctionMapper;
import com.persagy.apm.report.userconfig.function.service.IFunctionService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.apache.commons.lang.StringUtils;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.apm.report.userconfig.function.model.*;
import com.persagy.apm.report.userconfig.function.model.dto.*;

import java.util.List;
import java.util.Map;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.model.dto.Sort;
import org.springframework.util.CollectionUtils;

/**
 * 功能点(Function) service层
 *
 * @author lixing
 * @version V1.0 2021-07-23 17:55:17
 */
@Service
public class FunctionServiceImpl extends ServiceImpl<FunctionMapper, Function>
        implements IFunctionService {
    @Autowired
    FunctionMapper functionMapper;

    /**
     * 创建功能点
     *
     * @return 功能点主键
     * @author lixing
     * @version V1.0 2021-07-23 17:55:17
     */
    @Override
    public String createFunction(AddFunctionDTO addFunctionDTO) {
        Function function = ConvertFunctionTool.INSTANCE.convertAddDto2Entity(addFunctionDTO);
        // 设置默认值
        setDefaultValue(function);
        save(function);
        return function.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param function 功能点实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(Function function) {
        function.setCreator(AppContext.getContext().getAccountId());
        // todo 其他默认的属性

    }

    /**
     * 功能点详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-07-23 17:55:17
     */
    @Override
    public Function queryFunctionDetail(String id) {
        Function function = getById(id);
        if (function == null) {
            throw new IllegalArgumentException("查看Function详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return function;
    }

    /**
     * 更新功能点
     *
     * @author lixing
     * @version V1.0 2021-07-23 17:55:17
     */
    @Override
    public void updateFunction(UpdateFunctionDTO updateFunctionDTO) {
        Function function = getById(updateFunctionDTO.getId());
        function = ConvertFunctionTool.INSTANCE.convertUpdateDto2Entity(function, updateFunctionDTO);
        function.setModifier(AppContext.getContext().getAccountId());
        updateById(function);
    }

    /**
     * 校验功能点是否可删除
     *
     * @param id 功能点主键
     * @return 功能点do类
     * @author lixing
     * @version V1.0 2021-07-23 17:55:17
     */
    public Function checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除Function时发生异常，主键为空");
        }

        Function function = getById(id);

        if (function == null) {
            throw new IllegalArgumentException("删除Function时发生异常，找不到要删除的数据，id:" + id);
        }

        return function;
    }

    /**
     * 删除功能点
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-07-23 17:55:17
     */
    @Override
    public void deleteFunction(String id) {
        // 校验是否可删除
        Function function = checkDeletable(id);

        function.setValid(ValidEnum.FALSE.getType());
        updateById(function);
    }

    /**
     * 查询功能点
     *
     * @return List<Function>
     * @author lixing
     * @version V1.0 2021-07-23 17:55:17
     */
    @Override
    public List<Function> queryFunctionList(QueryFunctionDTO queryFunctionDTO) {
        QueryWrapper<Function> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(Function.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, Function.PROP_CREATIONTIME);

        if (queryFunctionDTO != null) {

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryFunctionDTO.getName())) {
                queryWrapper.like(Function.PROP_NAME, queryFunctionDTO.getName());
            }

            if (queryFunctionDTO.getDefine() != null) {
                queryWrapper.eq(Function.PROP_DEFINE, queryFunctionDTO.getDefine());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryFunctionDTO.getDataSource())) {
                queryWrapper.like(Function.PROP_DATA_SOURCE, queryFunctionDTO.getDataSource());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryFunctionDTO.getQualifiedStandard())) {
                queryWrapper.like(Function.PROP_QUALIFIED_STANDARD, queryFunctionDTO.getQualifiedStandard());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryFunctionDTO.getRemark())) {
                queryWrapper.like(Function.PROP_REMARK, queryFunctionDTO.getRemark());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询功能点
     *
     * @return IPage<Function>
     * @author lixing
     * @version V1.0 2021-07-23 17:55:17
     */
    @Override
    public IPage<Function> pageQueryFunction(PageQueryFunctionDTO pageQueryFunctionDTO) {
        QueryWrapper<Function> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(Function.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<Function> pageParam = new Page<>(pageQueryFunctionDTO.getPage(), pageQueryFunctionDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryFunctionDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, Function.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryFunctionDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryFunctionDTO.getName())) {
            queryWrapper.like(Function.PROP_NAME, pageQueryFunctionDTO.getName());
        }

        if (pageQueryFunctionDTO.getDefine() != null) {
            queryWrapper.eq(Function.PROP_DEFINE, pageQueryFunctionDTO.getDefine());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryFunctionDTO.getDataSource())) {
            queryWrapper.like(Function.PROP_DATA_SOURCE, pageQueryFunctionDTO.getDataSource());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryFunctionDTO.getQualifiedStandard())) {
            queryWrapper.like(Function.PROP_QUALIFIED_STANDARD, pageQueryFunctionDTO.getQualifiedStandard());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryFunctionDTO.getRemark())) {
            queryWrapper.like(Function.PROP_REMARK, pageQueryFunctionDTO.getRemark());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }

    @Override
    public List<Function> queryByDefine(String dataSource, Map<String, Object> defineMap) {
        return functionMapper.queryByDefine(dataSource, defineMap);
    }
}
