package com.persagy.apm.report.userconfig.paragraph.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.persagy.apm.common.context.AppContext;
import com.persagy.apm.common.model.dto.Sort;
import com.persagy.apm.report.common.utils.ApmStrUtils;
import com.persagy.apm.report.userconfig.paragraph.dao.ParagraphMapper;
import com.persagy.apm.report.userconfig.paragraph.model.ConvertParagraphTool;
import com.persagy.apm.report.userconfig.paragraph.model.Paragraph;
import com.persagy.apm.report.userconfig.paragraph.model.dto.AddParagraphDTO;
import com.persagy.apm.report.userconfig.paragraph.model.dto.PageQueryParagraphDTO;
import com.persagy.apm.report.userconfig.paragraph.model.dto.QueryParagraphDTO;
import com.persagy.apm.report.userconfig.paragraph.model.dto.UpdateParagraphDTO;
import com.persagy.apm.report.userconfig.paragraph.service.IParagraphService;
import com.persagy.apm.report.userconfig.pargraphdefine.model.ParagraphDefine;
import com.persagy.apm.report.userconfig.pargraphdefine.service.IParagraphDefineService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;

/**
 * 段落(Paragraph) service层
 *
 * @author lixing
 * @version V1.0 2021-07-15 17:46:12
 */
@Service
public class ParagraphServiceImpl extends ServiceImpl<ParagraphMapper, Paragraph>
        implements IParagraphService {
    @Autowired
    IParagraphDefineService paragraphDefineService;

    /**
     * 创建段落
     *
     * @return 段落主键
     * @author lixing
     * @version V1.0 2021-07-15 17:46:12
     */
    @Override
    public String createParagraph(AddParagraphDTO addParagraphDTO) {
        Paragraph paragraph = ConvertParagraphTool.INSTANCE.convertAddDto2Entity(addParagraphDTO);
        // 设置默认值
        setDefaultValue(paragraph);
        save(paragraph);
        return paragraph.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param paragraph 段落实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(Paragraph paragraph) {
        paragraph.setCreator(AppContext.getContext().getAccountId());
        // todo 其他默认的属性

    }

    /**
     * 段落详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-07-15 17:46:12
     */
    @Override
    public Paragraph queryParagraphDetail(String id) {
        Paragraph paragraph = getById(id);
        if (paragraph == null) {
            throw new IllegalArgumentException("查看Paragraph详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return paragraph;
    }

    /**
     * 更新段落
     *
     * @author lixing
     * @version V1.0 2021-07-15 17:46:12
     */
    @Override
    public void updateParagraph(UpdateParagraphDTO updateParagraphDTO) {
        Paragraph paragraph = getById(updateParagraphDTO.getId());
        paragraph = ConvertParagraphTool.INSTANCE.convertUpdateDto2Entity(paragraph, updateParagraphDTO);
        paragraph.setModifier(AppContext.getContext().getAccountId());
        updateById(paragraph);
    }

    /**
     * 校验段落是否可删除
     *
     * @param id 段落主键
     * @return 段落do类
     * @author lixing
     * @version V1.0 2021-07-15 17:46:12
     */
    public Paragraph checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除Paragraph时发生异常，主键为空");
        }

        Paragraph paragraph = getById(id);

        if (paragraph == null) {
            throw new IllegalArgumentException("删除Paragraph时发生异常，找不到要删除的数据，id:" + id);
        }

        return paragraph;
    }

    /**
     * 删除段落
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-07-15 17:46:12
     */
    @Override
    public void deleteParagraph(String id) {
        // 校验是否可删除
        Paragraph paragraph = checkDeletable(id);

        paragraph.setValid(ValidEnum.FALSE.getType());
        updateById(paragraph);
    }

    /**
     * 查询段落
     *
     * @return List<Paragraph>
     * @author lixing
     * @version V1.0 2021-07-15 17:46:12
     */
    @Override
    public List<Paragraph> queryParagraphList(QueryParagraphDTO queryParagraphDTO) {
        QueryWrapper<Paragraph> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(Paragraph.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, Paragraph.PROP_CREATIONTIME);

        if (queryParagraphDTO != null) {

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryParagraphDTO.getName())) {
                queryWrapper.like(Paragraph.PROP_NAME, queryParagraphDTO.getName());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryParagraphDTO.getDefineId())) {
                queryWrapper.like(Paragraph.PROP_DEFINE_ID, queryParagraphDTO.getDefineId());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryParagraphDTO.getCustomId())) {
                queryWrapper.like(Paragraph.PROP_CUSTOM_ID, queryParagraphDTO.getCustomId());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询段落
     *
     * @return IPage<Paragraph>
     * @author lixing
     * @version V1.0 2021-07-15 17:46:12
     */
    @Override
    public IPage<Paragraph> pageQueryParagraph(PageQueryParagraphDTO pageQueryParagraphDTO) {
        QueryWrapper<Paragraph> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(Paragraph.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<Paragraph> pageParam = new Page<>(pageQueryParagraphDTO.getPage(), pageQueryParagraphDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryParagraphDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, Paragraph.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryParagraphDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryParagraphDTO.getName())) {
            queryWrapper.like(Paragraph.PROP_NAME, pageQueryParagraphDTO.getName());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryParagraphDTO.getDefineId())) {
            queryWrapper.like(Paragraph.PROP_DEFINE_ID, pageQueryParagraphDTO.getDefineId());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryParagraphDTO.getCustomId())) {
            queryWrapper.like(Paragraph.PROP_CUSTOM_ID, pageQueryParagraphDTO.getCustomId());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }

    @Override
    public String getModelCode(String paragraphId) {
        Paragraph paragraph = queryParagraphDetail(paragraphId);
        return getModelCode(paragraph);
    }

    @Override
    public String getModelCode(Paragraph paragraph) {
        String defineId = paragraph.getDefineId();
        return paragraphDefineService.getModelCode(defineId);
    }

    @Override
    public List<String> getRelItemIds(Paragraph paragraph) {
        String defineId = paragraph.getDefineId();
        ParagraphDefine paragraphDefine = paragraphDefineService.queryParagraphDefineDetail(defineId);
        String itemIds = paragraphDefine.getItemIds();
        return ApmStrUtils.str2List(itemIds);
    }
}
